{===============================================================================
Same as demo 1&2 but auto-loading of modules
--------------------------------------------------------------------------------
 2001 Infersys,  D. Oldfield
===============================================================================}
unit MainF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TfrmMain = class(TForm)
    btnShowForm: TButton;
    lstModules: TListBox;
    btnClose: TButton;
    procedure btnCloseClick(Sender: TObject);
    procedure btnShowFormClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FHandles : TList;
    procedure LoadModule(aFile: string);
    procedure UnloadModules;
    procedure LoadModules;
  public

  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FHandles := TList.Create;
  LoadModules;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	UnloadModules;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  FHandles.Free;
end;

{------------------------------------------------------------------------------}
{ Button events }

procedure TfrmMain.btnShowFormClick(Sender: TObject);
var
  aClass: TPersistentClass;
  aForm : TForm;
begin
  aClass := GetClass('TfrmPlugIn');	//Must be registered to succeed

  if Assigned(aClass)then
  begin
    aForm := TFormClass(AClass).Create(Application);
    aForm.Show;
  end;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
	Close;
end;

{------------------------------------------------------------------------------}
{ Module methods }

procedure TfrmMain.LoadModules;
{ Automatically fetch modules }
var
	aFile: string;
  aSRec: TSearchRec;
begin
	aFile := ExtractFilePath(Application.ExeName) + '*.bpl';

  if (FindFirst(aFile, faAnyFile, aSRec) = 0) then
  try
  	repeat
  		LoadModule(aSRec.Name);
    until (FindNext(aSRec) > 0)
  finally
    FindClose(aSRec);
  end;
end;

procedure TfrmMain.LoadModule(aFile: string);
var
  Module: THandle;
  Descrip: string;
begin
  Module := LoadPackage(aFile); 	//LoadPackage returns a handle
	FHandles.Add(Pointer(Module));  //store reference to module

{ Get 'nice' name this time }
  Descrip := GetPackageDescription(PChar(aFile));
  lstModules.Items.Add(Descrip);
end;

procedure TfrmMain.UnloadModules;

  procedure FreeModuleComponents(aModule: THandle);
  var
		Loop : Integer;
    M: TMemoryBasicInformation;
  begin
  	with Application do
    begin
      for Loop := ComponentCount - 1 downto 0 do
      begin
        VirtualQuery(Components[Loop].ClassType, M, SizeOf(M));
        if (aModule = 0) or (HMODULE(M.AllocationBase) = aModule) then
          Components[Loop].Free;
      end;
    end;
	end;

var
	Loop : Integer;
  Module: THandle;

begin
	for Loop := FHandles.Count - 1 downto 0 do
  begin
  	Module := THandle(FHandles[Loop]);

		FreeModuleComponents(Module);  //destroy all created forms

  	UnRegisterModuleClasses(Module);
    UnLoadPackage(Module);
  end;
end;

END.
