{===============================================================================
Demo 4
	Module registers actions - see RegisterModule procedure
	Modules clear up rather than program - see change to UnloadModules
--------------------------------------------------------------------------------
 2001 Infersys,  D. Oldfield
===============================================================================}
unit MainF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, ToolWin, ComCtrls, ExtCtrls;

type
  TfrmMain = class(TForm)
    MainMenu: TMainMenu;
    mnuFile: TMenuItem;
    mnuExit: TMenuItem;
    pnlToolbar: TPanel;
    procedure CloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FHandles: TList;
    FModules: TList;
    procedure LoadModule(aFile: string);
    procedure UnloadModules;
    procedure LoadModules;
    procedure RegisterModule(aDescrip: string);
  public

  end;

var
  frmMain: TfrmMain;

implementation

uses Buttons, ActnList, ModuleSupportU;

{$R *.DFM}

procedure TfrmMain.FormCreate(Sender: TObject);
{ Could use a Contnrs list eg TComponentList for FModules
	then could free the list in UnloadModules procedure }
begin
  FHandles := TList.Create;
  FModules := TList.Create; 
  LoadModules;
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
	UnloadModules;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  FHandles.Free;
  FModules.Free;
end;

{------------------------------------------------------------------------------}
{ Menu events }

procedure TfrmMain.CloseClick(Sender: TObject);
begin
	Close;
end;

{------------------------------------------------------------------------------}
{ Module methods }

procedure TfrmMain.LoadModules;
var
	aFile: string;
  aSRec: TSearchRec;
begin
	aFile := ExtractFilePath(Application.ExeName) + 'P*.bpl';

  if (FindFirst(aFile, faAnyFile, aSRec) = 0) then
  try
  	repeat
  		LoadModule(aSRec.Name);
    until (FindNext(aSRec) > 0)
  finally
    FindClose(aSRec);
  end;
end;

procedure TfrmMain.LoadModule(aFile: string);
var
  Module: THandle;
  Descrip: string;
begin
  Module := LoadPackage(aFile); 	//LoadPackage returns a handle
	FHandles.Add(Pointer(Module));  //store reference to plug-in

  Descrip := GetPackageDescription(PChar(aFile));

  RegisterModule(Descrip);
end;

procedure TfrmMain.RegisterModule(aDescrip: string);

  procedure AddButtons(aList: TActionList);
  var
  	Loop: Integer;
  begin
  	for Loop := 0 to aList.ActionCount - 1 do
  		with TSpeedButton.Create(pnlToolBar) do
    	begin
    		Parent := pnlToolbar;
        Align  := alLeft;
        Width  := 100;
      	Action := aList.Actions[Loop];
    	end;
  end;

var
	aClass: TModuleClass;
  aModule: TModule;
  aList: TActionList;
  
begin
	aClass := TModuleClass(GetClass(aDescrip));

  if Assigned(aClass) then
  begin
  	aModule := aClass.Create(Self);
    FModules.Add(aModule);
    
    aList := aModule.GetFormActions;
    AddButtons(aList)
  end;
end;

procedure TfrmMain.UnloadModules;

  procedure FreeModules;
  var
  	Loop: Integer;
  begin
    for Loop := 0 to FModules.Count - 1 do
    	TModule(FModules[Loop]).Free;
    FModules.Clear;
  end;

var
	Loop : Integer;
  aModule: THandle;

begin
  FreeModules; //free Module instances

	for Loop := FHandles.Count - 1 downto 0 do
  begin
  	aModule := THandle(FHandles[Loop]);
  	UnRegisterModuleClasses(aModule);
    UnLoadPackage(aModule);
  end;
end;

END.
